package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickIMUV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickIMUV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Orientation")
    public @ActionOutput(name = "heading", type="short")
           @ActionOutput(name = "roll", type="short")
           @ActionOutput(name = "pitch", type="short")
           Map<String, Object> brickIMUV2GetOrientation(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.Orientation value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getOrientation();
        
        result.put("heading", value.heading);
        result.put("roll", value.roll);
        result.put("pitch", value.pitch);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetOrientation(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetOrientation();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Linear Acceleration")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetLinearAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.LinearAcceleration value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getLinearAcceleration();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetLinearAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetLinearAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Gravity Vector")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetGravityVector(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.GravityVector value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getGravityVector();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetGravityVector(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetGravityVector();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Quaternion")
    public @ActionOutput(name = "w", type="short")
           @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetQuaternion(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.Quaternion value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getQuaternion();
        
        result.put("w", value.w);
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetQuaternion(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetQuaternion();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get All Data")
    public @ActionOutput(name = "acceleration", type="short[]")
           @ActionOutput(name = "magneticField", type="short[]")
           @ActionOutput(name = "angularVelocity", type="short[]")
           @ActionOutput(name = "eulerAngle", type="short[]")
           @ActionOutput(name = "quaternion", type="short[]")
           @ActionOutput(name = "linearAcceleration", type="short[]")
           @ActionOutput(name = "gravityVector", type="short[]")
           @ActionOutput(name = "temperature", type="byte")
           @ActionOutput(name = "calibrationStatus", type="short")
           Map<String, Object> brickIMUV2GetAllData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.AllData value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getAllData();
        
        result.put("acceleration", value.acceleration);
        result.put("magneticField", value.magneticField);
        result.put("angularVelocity", value.angularVelocity);
        result.put("eulerAngle", value.eulerAngle);
        result.put("quaternion", value.quaternion);
        result.put("linearAcceleration", value.linearAcceleration);
        result.put("gravityVector", value.gravityVector);
        result.put("temperature", value.temperature);
        result.put("calibrationStatus", value.calibrationStatus);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetAllData(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetAllData();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Leds On")
    public void brickIMUV2LedsOn(
            ) throws TinkerforgeException {
        ((BrickIMUV2Wrapper)this.handler.getDevice()).ledsOn();
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickIMUV2EnableLEDs")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickIMUV2EnableLEDs"), RefreshType.REFRESH);
}
    }

    public static void brickIMUV2LedsOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            ((BrickIMUV2Actions) actions).brickIMUV2LedsOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Leds Off")
    public void brickIMUV2LedsOff(
            ) throws TinkerforgeException {
        ((BrickIMUV2Wrapper)this.handler.getDevice()).ledsOff();
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickIMUV2EnableLEDs")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickIMUV2EnableLEDs"), RefreshType.REFRESH);
}
    }

    public static void brickIMUV2LedsOff(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            ((BrickIMUV2Actions) actions).brickIMUV2LedsOff();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.Acceleration value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getAcceleration();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Magnetic Field")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetMagneticField(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.MagneticField value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getMagneticField();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetMagneticField(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetMagneticField();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Angular Velocity")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickIMUV2GetAngularVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.AngularVelocity value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getAngularVelocity();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetAngularVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetAngularVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="byte")
           Map<String, Object> brickIMUV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        byte value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensor Configuration")
    public @ActionOutput(name = "magnetometerRate", type="short")
           @ActionOutput(name = "gyroscopeRange", type="short")
           @ActionOutput(name = "gyroscopeBandwidth", type="short")
           @ActionOutput(name = "accelerometerRange", type="short")
           @ActionOutput(name = "accelerometerBandwidth", type="short")
           Map<String, Object> brickIMUV2GetSensorConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.SensorConfiguration value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSensorConfiguration();
        
        result.put("magnetometerRate", value.magnetometerRate);
        result.put("gyroscopeRange", value.gyroscopeRange);
        result.put("gyroscopeBandwidth", value.gyroscopeBandwidth);
        result.put("accelerometerRange", value.accelerometerRange);
        result.put("accelerometerBandwidth", value.accelerometerBandwidth);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSensorConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSensorConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensor Fusion Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> brickIMUV2GetSensorFusionMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSensorFusionMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSensorFusionMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSensorFusionMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickIMUV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.Identity value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickIMUV2IsStatusLEDEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickIMUV2Wrapper)this.handler.getDevice()).isStatusLEDEnabled();
        
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2IsStatusLEDEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2IsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="short")
           Map<String, Object> brickIMUV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type="long")
           Map<String, Object> brickIMUV2GetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") short communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSendTimeoutCount(communicationMethod);
        
        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSendTimeoutCount(@Nullable ThingActions actions, short communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type="boolean")
           @ActionOutput(name = "minimumDynamicBaudrate", type="long")
           Map<String, Object> brickIMUV2GetSPITFPBaudrateConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.SPITFPBaudrateConfig value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSPITFPBaudrateConfig();
        
        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSPITFPBaudrateConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickIMUV2GetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMUV2.SPITFPErrorCount value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount(brickletPort);
        
        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type="long")
           Map<String, Object> brickIMUV2GetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickIMUV2Wrapper)this.handler.getDevice()).getSPITFPBaudrate(brickletPort);
        
        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickIMUV2GetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickIMUV2Actions) {
            return ((BrickIMUV2Actions) actions).brickIMUV2GetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUV2Actions class.");
        }
    }
}
