package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAccelerometerV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAccelerometerV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletAccelerometerV2GetAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometerV2.Acceleration value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getAcceleration();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "dataRate", type="int")
           @ActionOutput(name = "fullScale", type="int")
           Map<String, Object> brickletAccelerometerV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometerV2.Configuration value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getConfiguration();
        
        result.put("dataRate", value.dataRate);
        result.put("fullScale", value.fullScale);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Info LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletAccelerometerV2GetInfoLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getInfoLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetInfoLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetInfoLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Filter Configuration")
    public @ActionOutput(name = "iirBypass", type="int")
           @ActionOutput(name = "lowPassFilter", type="int")
           Map<String, Object> brickletAccelerometerV2GetFilterConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometerV2.FilterConfiguration value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getFilterConfiguration();
        
        result.put("iirBypass", value.iirBypass);
        result.put("lowPassFilter", value.lowPassFilter);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetFilterConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetFilterConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletAccelerometerV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletAccelerometerV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletAccelerometerV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletAccelerometerV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletAccelerometerV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometerV2.SPITFPErrorCount value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAccelerometerV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometerV2.Identity value = ((BrickletAccelerometerV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerV2Actions) {
            return ((BrickletAccelerometerV2Actions) actions).brickletAccelerometerV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerV2Actions class.");
        }
    }
}
