package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAirQuality;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAirQualityActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get All Values")
    public @ActionOutput(name = "iaqIndex", type="int")
           @ActionOutput(name = "iaqIndexAccuracy", type="int")
           @ActionOutput(name = "temperature", type="int")
           @ActionOutput(name = "humidity", type="int")
           @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletAirQualityGetAllValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAirQuality.AllValues value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getAllValues();
        
        result.put("iaqIndex", value.iaqIndex);
        result.put("iaqIndexAccuracy", value.iaqIndexAccuracy);
        result.put("temperature", value.temperature);
        result.put("humidity", value.humidity);
        result.put("airPressure", value.airPressure);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetAllValues(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetAllValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Temperature Offset")
    public @ActionOutput(name = "offset", type="int")
           Map<String, Object> brickletAirQualityGetTemperatureOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getTemperatureOffset();
        
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetTemperatureOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetTemperatureOffset();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get IAQ Index")
    public @ActionOutput(name = "iaqIndex", type="int")
           @ActionOutput(name = "iaqIndexAccuracy", type="int")
           Map<String, Object> brickletAirQualityGetIAQIndex(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAirQuality.IAQIndex value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getIAQIndex();
        
        result.put("iaqIndex", value.iaqIndex);
        result.put("iaqIndexAccuracy", value.iaqIndexAccuracy);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetIAQIndex(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetIAQIndex();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletAirQualityGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Humidity")
    public @ActionOutput(name = "humidity", type="int")
           Map<String, Object> brickletAirQualityGetHumidity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getHumidity();
        
        result.put("humidity", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetHumidity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetHumidity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletAirQualityGetAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getAirPressure();
        
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Background Calibration Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> brickletAirQualityGetBackgroundCalibrationDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getBackgroundCalibrationDuration();
        
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetBackgroundCalibrationDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetBackgroundCalibrationDuration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletAirQualityReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletAirQualityWrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletAirQualityGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletAirQualityGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletAirQualityGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletAirQualityGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAirQuality.SPITFPErrorCount value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAirQualityGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAirQuality.Identity value = ((BrickletAirQualityWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAirQualityGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAirQualityActions) {
            return ((BrickletAirQualityActions) actions).brickletAirQualityGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAirQualityActions class.");
        }
    }
}
