package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAnalogInV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogInV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogInV2GetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV2Wrapper)this.handler.getDevice()).getVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV2GetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV2Actions) {
            return ((BrickletAnalogInV2Actions) actions).brickletAnalogInV2GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletAnalogInV2GetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV2Wrapper)this.handler.getDevice()).getAnalogValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV2GetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV2Actions) {
            return ((BrickletAnalogInV2Actions) actions).brickletAnalogInV2GetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="short")
           Map<String, Object> brickletAnalogInV2GetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletAnalogInV2Wrapper)this.handler.getDevice()).getMovingAverage();
        
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV2GetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV2Actions) {
            return ((BrickletAnalogInV2Actions) actions).brickletAnalogInV2GetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogInV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogInV2.Identity value = ((BrickletAnalogInV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV2Actions) {
            return ((BrickletAnalogInV2Actions) actions).brickletAnalogInV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV2Actions class.");
        }
    }
}
