package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAnalogOutV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Output Voltage")
    public void brickletAnalogOutV2SetOutputVoltage(
            @ActionInput(name = "voltage") int voltage) throws TinkerforgeException {
        ((BrickletAnalogOutV2Wrapper)this.handler.getDevice()).setOutputVoltage(voltage);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutV2OutputVoltage")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutV2OutputVoltage"), RefreshType.REFRESH);
}
    }

    public static void brickletAnalogOutV2SetOutputVoltage(@Nullable ThingActions actions, int voltage) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            ((BrickletAnalogOutV2Actions) actions).brickletAnalogOutV2SetOutputVoltage(voltage);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV2GetOutputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV2Wrapper)this.handler.getDevice()).getOutputVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetOutputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            return ((BrickletAnalogOutV2Actions) actions).brickletAnalogOutV2GetOutputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV2GetInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV2Wrapper)this.handler.getDevice()).getInputVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            return ((BrickletAnalogOutV2Actions) actions).brickletAnalogOutV2GetInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogOutV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogOutV2.Identity value = ((BrickletAnalogOutV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            return ((BrickletAnalogOutV2Actions) actions).brickletAnalogOutV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }
}
