package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletCANV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCANV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write Frame")
    public @ActionOutput(name = "success", type="boolean")
           Map<String, Object> brickletCANV2WriteFrame(
            @ActionInput(name = "frameType") int frameType,
            @ActionInput(name = "identifier") long identifier,
            @ActionInput(name = "data") int[] data) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletCANV2Wrapper)this.handler.getDevice()).writeFrame(frameType, identifier, data);
        
        result.put("success", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2WriteFrame(@Nullable ThingActions actions, int frameType, long identifier, int[] data) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2WriteFrame(frameType, identifier, data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Read Frame")
    public @ActionOutput(name = "success", type="boolean")
           @ActionOutput(name = "frameType", type="int")
           @ActionOutput(name = "identifier", type="long")
           @ActionOutput(name = "data", type="int[]")
           Map<String, Object> brickletCANV2ReadFrame(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.ReadFrame value = ((BrickletCANV2Wrapper)this.handler.getDevice()).readFrame();
        
        result.put("success", value.success);
        result.put("frameType", value.frameType);
        result.put("identifier", value.identifier);
        result.put("data", value.data);
        return result;
    }

    public static Map<String, Object> brickletCANV2ReadFrame(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2ReadFrame();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Transceiver Configuration")
    public @ActionOutput(name = "baudRate", type="long")
           @ActionOutput(name = "samplePoint", type="int")
           @ActionOutput(name = "transceiverMode", type="int")
           Map<String, Object> brickletCANV2GetTransceiverConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.TransceiverConfiguration value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getTransceiverConfiguration();
        
        result.put("baudRate", value.baudRate);
        result.put("samplePoint", value.samplePoint);
        result.put("transceiverMode", value.transceiverMode);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetTransceiverConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetTransceiverConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Set Queue Configuration")
    public void brickletCANV2SetQueueConfiguration(
            @ActionInput(name = "writeBufferSize") int writeBufferSize,
            @ActionInput(name = "writeBufferTimeout") int writeBufferTimeout,
            @ActionInput(name = "writeBacklogSize") int writeBacklogSize,
            @ActionInput(name = "readBufferSizes") int[] readBufferSizes,
            @ActionInput(name = "readBacklogSize") int readBacklogSize) throws TinkerforgeException {
        ((BrickletCANV2Wrapper)this.handler.getDevice()).setQueueConfiguration(writeBufferSize, writeBufferTimeout, writeBacklogSize, readBufferSizes, readBacklogSize);
        
    }

    public static void brickletCANV2SetQueueConfiguration(@Nullable ThingActions actions, int writeBufferSize, int writeBufferTimeout, int writeBacklogSize, int[] readBufferSizes, int readBacklogSize) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            ((BrickletCANV2Actions) actions).brickletCANV2SetQueueConfiguration(writeBufferSize, writeBufferTimeout, writeBacklogSize, readBufferSizes, readBacklogSize);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Queue Configuration")
    public @ActionOutput(name = "writeBufferSize", type="int")
           @ActionOutput(name = "writeBufferTimeout", type="int")
           @ActionOutput(name = "writeBacklogSize", type="int")
           @ActionOutput(name = "readBufferSizes", type="int[]")
           @ActionOutput(name = "readBacklogSize", type="int")
           Map<String, Object> brickletCANV2GetQueueConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.QueueConfiguration value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getQueueConfiguration();
        
        result.put("writeBufferSize", value.writeBufferSize);
        result.put("writeBufferTimeout", value.writeBufferTimeout);
        result.put("writeBacklogSize", value.writeBacklogSize);
        result.put("readBufferSizes", value.readBufferSizes);
        result.put("readBacklogSize", value.readBacklogSize);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetQueueConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetQueueConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Set Read Filter Configuration")
    public void brickletCANV2SetReadFilterConfiguration(
            @ActionInput(name = "bufferIndex") int bufferIndex,
            @ActionInput(name = "filterMode") int filterMode,
            @ActionInput(name = "filterMask") long filterMask,
            @ActionInput(name = "filterIdentifier") long filterIdentifier) throws TinkerforgeException {
        ((BrickletCANV2Wrapper)this.handler.getDevice()).setReadFilterConfiguration(bufferIndex, filterMode, filterMask, filterIdentifier);
        
    }

    public static void brickletCANV2SetReadFilterConfiguration(@Nullable ThingActions actions, int bufferIndex, int filterMode, long filterMask, long filterIdentifier) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            ((BrickletCANV2Actions) actions).brickletCANV2SetReadFilterConfiguration(bufferIndex, filterMode, filterMask, filterIdentifier);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Read Filter Configuration")
    public @ActionOutput(name = "filterMode", type="int")
           @ActionOutput(name = "filterMask", type="long")
           @ActionOutput(name = "filterIdentifier", type="long")
           Map<String, Object> brickletCANV2GetReadFilterConfiguration(
            @ActionInput(name = "bufferIndex") int bufferIndex) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.ReadFilterConfiguration value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getReadFilterConfiguration(bufferIndex);
        
        result.put("filterMode", value.filterMode);
        result.put("filterMask", value.filterMask);
        result.put("filterIdentifier", value.filterIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetReadFilterConfiguration(@Nullable ThingActions actions, int bufferIndex) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetReadFilterConfiguration(bufferIndex);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Error Log")
    public @ActionOutput(name = "transceiverState", type="int")
           @ActionOutput(name = "transceiverWriteErrorLevel", type="int")
           @ActionOutput(name = "transceiverReadErrorLevel", type="int")
           @ActionOutput(name = "transceiverStuffingErrorCount", type="long")
           @ActionOutput(name = "transceiverFormatErrorCount", type="long")
           @ActionOutput(name = "transceiverACKErrorCount", type="long")
           @ActionOutput(name = "transceiverBit1ErrorCount", type="long")
           @ActionOutput(name = "transceiverBit0ErrorCount", type="long")
           @ActionOutput(name = "transceiverCRCErrorCount", type="long")
           @ActionOutput(name = "writeBufferTimeoutErrorCount", type="long")
           @ActionOutput(name = "readBufferOverflowErrorCount", type="long")
           @ActionOutput(name = "readBufferOverflowErrorOccurred", type="boolean[]")
           @ActionOutput(name = "readBacklogOverflowErrorCount", type="long")
           Map<String, Object> brickletCANV2GetErrorLog(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.ErrorLog value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getErrorLog();
        
        result.put("transceiverState", value.transceiverState);
        result.put("transceiverWriteErrorLevel", value.transceiverWriteErrorLevel);
        result.put("transceiverReadErrorLevel", value.transceiverReadErrorLevel);
        result.put("transceiverStuffingErrorCount", value.transceiverStuffingErrorCount);
        result.put("transceiverFormatErrorCount", value.transceiverFormatErrorCount);
        result.put("transceiverACKErrorCount", value.transceiverACKErrorCount);
        result.put("transceiverBit1ErrorCount", value.transceiverBit1ErrorCount);
        result.put("transceiverBit0ErrorCount", value.transceiverBit0ErrorCount);
        result.put("transceiverCRCErrorCount", value.transceiverCRCErrorCount);
        result.put("writeBufferTimeoutErrorCount", value.writeBufferTimeoutErrorCount);
        result.put("readBufferOverflowErrorCount", value.readBufferOverflowErrorCount);
        result.put("readBufferOverflowErrorOccurred", value.readBufferOverflowErrorOccurred);
        result.put("readBacklogOverflowErrorCount", value.readBacklogOverflowErrorCount);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetErrorLog(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetErrorLog();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletCANV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletCANV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletCANV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletCANV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletCANV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletCANV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.SPITFPErrorCount value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletCANV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.Identity value = ((BrickletCANV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }
}
