package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletColor;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletColorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Color")
    public @ActionOutput(name = "r", type="int")
           @ActionOutput(name = "g", type="int")
           @ActionOutput(name = "b", type="int")
           @ActionOutput(name = "c", type="int")
           Map<String, Object> brickletColorGetColor(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColor.Color value = ((BrickletColorWrapper)this.handler.getDevice()).getColor();
        
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("c", value.c);
        return result;
    }

    public static Map<String, Object> brickletColorGetColor(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorGetColor();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Light On")
    public void brickletColorLightOn(
            ) throws TinkerforgeException {
        ((BrickletColorWrapper)this.handler.getDevice()).lightOn();
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletColorLight")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletColorLight"), RefreshType.REFRESH);
}
    }

    public static void brickletColorLightOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            ((BrickletColorActions) actions).brickletColorLightOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Light Off")
    public void brickletColorLightOff(
            ) throws TinkerforgeException {
        ((BrickletColorWrapper)this.handler.getDevice()).lightOff();
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletColorLight")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletColorLight"), RefreshType.REFRESH);
}
    }

    public static void brickletColorLightOff(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            ((BrickletColorActions) actions).brickletColorLightOff();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Is Light On")
    public @ActionOutput(name = "light", type="short")
           Map<String, Object> brickletColorIsLightOn(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletColorWrapper)this.handler.getDevice()).isLightOn();
        
        result.put("light", value);
        return result;
    }

    public static Map<String, Object> brickletColorIsLightOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorIsLightOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Get Config")
    public @ActionOutput(name = "gain", type="short")
           @ActionOutput(name = "integrationTime", type="short")
           Map<String, Object> brickletColorGetConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColor.Config value = ((BrickletColorWrapper)this.handler.getDevice()).getConfig();
        
        result.put("gain", value.gain);
        result.put("integrationTime", value.integrationTime);
        return result;
    }

    public static Map<String, Object> brickletColorGetConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorGetConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type="long")
           Map<String, Object> brickletColorGetIlluminance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletColorWrapper)this.handler.getDevice()).getIlluminance();
        
        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletColorGetIlluminance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorGetIlluminance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Get Color Temperature")
    public @ActionOutput(name = "colorTemperature", type="int")
           Map<String, Object> brickletColorGetColorTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletColorWrapper)this.handler.getDevice()).getColorTemperature();
        
        result.put("colorTemperature", value);
        return result;
    }

    public static Map<String, Object> brickletColorGetColorTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorGetColorTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletColorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColor.Identity value = ((BrickletColorWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletColorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorActions) {
            return ((BrickletColorActions) actions).brickletColorGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorActions class.");
        }
    }
}
