package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDistanceIR;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDistanceIRActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type="int")
           Map<String, Object> brickletDistanceIRGetDistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRWrapper)this.handler.getDevice()).getDistance();
        
        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetDistance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletDistanceIRGetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRWrapper)this.handler.getDevice()).getAnalogValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Sampling Point")
    public @ActionOutput(name = "distance", type="int")
           Map<String, Object> brickletDistanceIRGetSamplingPoint(
            @ActionInput(name = "position") short position) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRWrapper)this.handler.getDevice()).getSamplingPoint(position);
        
        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetSamplingPoint(@Nullable ThingActions actions, short position) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetSamplingPoint(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDistanceIRGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDistanceIR.Identity value = ((BrickletDistanceIRWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }
}
