package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDualRelay;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set State")
    public void brickletDualRelaySetState(
            @ActionInput(name = "relay1") boolean relay1,
            @ActionInput(name = "relay2") boolean relay2) throws TinkerforgeException {
        ((BrickletDualRelayWrapper)this.handler.getDevice()).setState(relay1, relay2);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
}
    }

    public static void brickletDualRelaySetState(@Nullable ThingActions actions, boolean relay1, boolean relay2) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetState(relay1, relay2);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Set Selected State")
    public void brickletDualRelaySetSelectedState(
            @ActionInput(name = "relay") short relay,
            @ActionInput(name = "state") boolean state) throws TinkerforgeException {
        ((BrickletDualRelayWrapper)this.handler.getDevice()).setSelectedState(relay, state);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
}
    }

    public static void brickletDualRelaySetSelectedState(@Nullable ThingActions actions, short relay, boolean state) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetSelectedState(relay, state);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletDualRelaySetMonoflop(
            @ActionInput(name = "relay") short relay,
            @ActionInput(name = "state") boolean state,
            @ActionInput(name = "time") long time) throws TinkerforgeException {
        ((BrickletDualRelayWrapper)this.handler.getDevice()).setMonoflop(relay, state, time);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
}
    }

    public static void brickletDualRelaySetMonoflop(@Nullable ThingActions actions, short relay, boolean state, long time) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetMonoflop(relay, state, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "relay1", type="boolean")
           @ActionOutput(name = "relay2", type="boolean")
           Map<String, Object> brickletDualRelayGetState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualRelay.State value = ((BrickletDualRelayWrapper)this.handler.getDevice()).getState();
        
        result.put("relay1", value.relay1);
        result.put("relay2", value.relay2);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletDualRelayGetMonoflop(
            @ActionInput(name = "relay") short relay) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualRelay.Monoflop value = ((BrickletDualRelayWrapper)this.handler.getDevice()).getMonoflop(relay);
        
        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetMonoflop(@Nullable ThingActions actions, short relay) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetMonoflop(relay);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDualRelayGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualRelay.Identity value = ((BrickletDualRelayWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }
}
