package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialQuadRelay;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialQuadRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Value")
    public void brickletIndustrialQuadRelaySetValue(
            @ActionInput(name = "valueMask") int valueMask) throws TinkerforgeException {
        ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).setValue(valueMask);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3"), RefreshType.REFRESH);
}
    }

    public static void brickletIndustrialQuadRelaySetValue(@Nullable ThingActions actions, int valueMask) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelaySetValue(valueMask);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletIndustrialQuadRelaySetMonoflop(
            @ActionInput(name = "selectionMask") int selectionMask,
            @ActionInput(name = "valueMask") int valueMask,
            @ActionInput(name = "time") long time) throws TinkerforgeException {
        ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).setMonoflop(selectionMask, valueMask, time);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3"), RefreshType.REFRESH);
}
    }

    public static void brickletIndustrialQuadRelaySetMonoflop(@Nullable ThingActions actions, int selectionMask, int valueMask, long time) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelaySetMonoflop(selectionMask, valueMask, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Set Selected Values")
    public void brickletIndustrialQuadRelaySetSelectedValues(
            @ActionInput(name = "selectionMask") int selectionMask,
            @ActionInput(name = "valueMask") int valueMask) throws TinkerforgeException {
        ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).setSelectedValues(selectionMask, valueMask);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayRelay3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialQuadRelayMonoflopRelay3"), RefreshType.REFRESH);
}
    }

    public static void brickletIndustrialQuadRelaySetSelectedValues(@Nullable ThingActions actions, int selectionMask, int valueMask) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelaySetSelectedValues(selectionMask, valueMask);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIndustrialQuadRelayGetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getValue();
        
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="int")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIndustrialQuadRelayGetMonoflop(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelay.Monoflop value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getMonoflop(pin);
        
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetMonoflop(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetMonoflop(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialQuadRelayGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelay.Identity value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }
}
