package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIsolator;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIsolatorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Statistics")
    public @ActionOutput(name = "messagesFromBrick", type="long")
           @ActionOutput(name = "messagesFromBricklet", type="long")
           @ActionOutput(name = "connectedBrickletDeviceIdentifier", type="int")
           @ActionOutput(name = "connectedBrickletUID", type="String")
           Map<String, Object> brickletIsolatorGetStatistics(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIsolator.Statistics value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getStatistics();
        
        result.put("messagesFromBrick", value.messagesFromBrick);
        result.put("messagesFromBricklet", value.messagesFromBricklet);
        result.put("connectedBrickletDeviceIdentifier", value.connectedBrickletDeviceIdentifier);
        result.put("connectedBrickletUID", value.connectedBrickletUID);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetStatistics(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetStatistics();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIsolatorReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIsolatorWrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIsolatorReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIsolatorGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIsolatorGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIsolatorGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIsolatorGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIsolator.SPITFPErrorCount value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIsolatorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIsolator.Identity value = ((BrickletIsolatorWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIsolatorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).brickletIsolatorGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }
}
