package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMoisture;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMoistureActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Moisture Value")
    public @ActionOutput(name = "moisture", type="int")
           Map<String, Object> brickletMoistureGetMoistureValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMoistureWrapper)this.handler.getDevice()).getMoistureValue();
        
        result.put("moisture", value);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetMoistureValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMoistureActions) {
            return ((BrickletMoistureActions) actions).brickletMoistureGetMoistureValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMoistureActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="short")
           Map<String, Object> brickletMoistureGetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletMoistureWrapper)this.handler.getDevice()).getMovingAverage();
        
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMoistureActions) {
            return ((BrickletMoistureActions) actions).brickletMoistureGetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMoistureActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMoistureGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMoisture.Identity value = ((BrickletMoistureWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMoistureActions) {
            return ((BrickletMoistureActions) actions).brickletMoistureGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMoistureActions class.");
        }
    }
}
