package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMotionDetectorV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotionDetectorV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Motion Detected")
    public @ActionOutput(name = "motion", type="int")
           Map<String, Object> brickletMotionDetectorV2GetMotionDetected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getMotionDetected();
        
        result.put("motion", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetMotionDetected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetMotionDetected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensitivity")
    public @ActionOutput(name = "sensitivity", type="int")
           Map<String, Object> brickletMotionDetectorV2GetSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getSensitivity();
        
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Set Indicator")
    public void brickletMotionDetectorV2SetIndicator(
            @ActionInput(name = "topLeft") int topLeft,
            @ActionInput(name = "topRight") int topRight,
            @ActionInput(name = "bottom") int bottom) throws TinkerforgeException {
        ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).setIndicator(topLeft, topRight, bottom);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2TopLeftIndicator")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2TopLeftIndicator"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2TopRightIndicator")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2TopRightIndicator"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2BottomIndicator")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletMotionDetectorV2BottomIndicator"), RefreshType.REFRESH);
}
    }

    public static void brickletMotionDetectorV2SetIndicator(@Nullable ThingActions actions, int topLeft, int topRight, int bottom) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2SetIndicator(topLeft, topRight, bottom);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Indicator")
    public @ActionOutput(name = "topLeft", type="int")
           @ActionOutput(name = "topRight", type="int")
           @ActionOutput(name = "bottom", type="int")
           Map<String, Object> brickletMotionDetectorV2GetIndicator(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotionDetectorV2.Indicator value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getIndicator();
        
        result.put("topLeft", value.topLeft);
        result.put("topRight", value.topRight);
        result.put("bottom", value.bottom);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIndicator(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetIndicator();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletMotionDetectorV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletMotionDetectorV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletMotionDetectorV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotionDetectorV2.SPITFPErrorCount value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMotionDetectorV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotionDetectorV2.Identity value = ((BrickletMotionDetectorV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }
}
