package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletPiezoBuzzer;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPiezoBuzzerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Beep")
    public void brickletPiezoBuzzerBeep(
            @ActionInput(name = "duration") long duration) throws TinkerforgeException {
        ((BrickletPiezoBuzzerWrapper)this.handler.getDevice()).beep(duration);
        
    }

    public static void brickletPiezoBuzzerBeep(@Nullable ThingActions actions, long duration) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoBuzzerActions) {
            ((BrickletPiezoBuzzerActions) actions).brickletPiezoBuzzerBeep(duration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoBuzzerActions class.");
        }
    }

    @RuleAction(label = "Morse Code")
    public void brickletPiezoBuzzerMorseCode(
            @ActionInput(name = "morse") String morse) throws TinkerforgeException {
        ((BrickletPiezoBuzzerWrapper)this.handler.getDevice()).morseCode(morse);
        
    }

    public static void brickletPiezoBuzzerMorseCode(@Nullable ThingActions actions, String morse) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoBuzzerActions) {
            ((BrickletPiezoBuzzerActions) actions).brickletPiezoBuzzerMorseCode(morse);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoBuzzerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPiezoBuzzerGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoBuzzer.Identity value = ((BrickletPiezoBuzzerWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPiezoBuzzerGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoBuzzerActions) {
            return ((BrickletPiezoBuzzerActions) actions).brickletPiezoBuzzerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoBuzzerActions class.");
        }
    }
}
