package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRGBLEDButton;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDButtonActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Color")
    public @ActionOutput(name = "red", type="int")
           @ActionOutput(name = "green", type="int")
           @ActionOutput(name = "blue", type="int")
           Map<String, Object> brickletRGBLEDButtonGetColor(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.Color value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getColor();
        
        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetColor(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetColor();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "state", type="int")
           Map<String, Object> brickletRGBLEDButtonGetButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getButtonState();
        
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Color Calibration")
    public @ActionOutput(name = "red", type="int")
           @ActionOutput(name = "green", type="int")
           @ActionOutput(name = "blue", type="int")
           Map<String, Object> brickletRGBLEDButtonGetColorCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.ColorCalibration value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getColorCalibration();
        
        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetColorCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetColorCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Set Color")
    public void brickletRGBLEDButtonSetColor(
            @ActionInput(name = "red") int red,
            @ActionInput(name = "green") int green,
            @ActionInput(name = "blue") int blue) throws TinkerforgeException {
        ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).setColor(red, green, blue);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletRGBLEDButtonColor")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletRGBLEDButtonColor"), RefreshType.REFRESH);
}
    }

    public static void brickletRGBLEDButtonSetColor(@Nullable ThingActions actions, int red, int green, int blue) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonSetColor(red, green, blue);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRGBLEDButtonReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRGBLEDButtonGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRGBLEDButtonGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRGBLEDButtonGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRGBLEDButtonGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.SPITFPErrorCount value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRGBLEDButtonGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.Identity value = ((BrickletRGBLEDButtonWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }
}
