package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRealTimeClock;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRealTimeClockActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Date Time")
    public void brickletRealTimeClockSetDateTime(
            @ActionInput(name = "year") int year,
            @ActionInput(name = "month") short month,
            @ActionInput(name = "day") short day,
            @ActionInput(name = "hour") short hour,
            @ActionInput(name = "minute") short minute,
            @ActionInput(name = "second") short second,
            @ActionInput(name = "centisecond") short centisecond,
            @ActionInput(name = "weekday") short weekday) throws TinkerforgeException {
        ((BrickletRealTimeClockWrapper)this.handler.getDevice()).setDateTime(year, month, day, hour, minute, second, centisecond, weekday);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletRealTimeClockDateTime")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletRealTimeClockDateTime"), RefreshType.REFRESH);
}
    }

    public static void brickletRealTimeClockSetDateTime(@Nullable ThingActions actions, int year, short month, short day, short hour, short minute, short second, short centisecond, short weekday) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            ((BrickletRealTimeClockActions) actions).brickletRealTimeClockSetDateTime(year, month, day, hour, minute, second, centisecond, weekday);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "year", type="int")
           @ActionOutput(name = "month", type="short")
           @ActionOutput(name = "day", type="short")
           @ActionOutput(name = "hour", type="short")
           @ActionOutput(name = "minute", type="short")
           @ActionOutput(name = "second", type="short")
           @ActionOutput(name = "centisecond", type="short")
           @ActionOutput(name = "weekday", type="short")
           Map<String, Object> brickletRealTimeClockGetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClock.DateTime value = ((BrickletRealTimeClockWrapper)this.handler.getDevice()).getDateTime();
        
        result.put("year", value.year);
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("centisecond", value.centisecond);
        result.put("weekday", value.weekday);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).brickletRealTimeClockGetDateTime();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Timestamp")
    public @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockGetTimestamp(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRealTimeClockWrapper)this.handler.getDevice()).getTimestamp();
        
        result.put("timestamp", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetTimestamp(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).brickletRealTimeClockGetTimestamp();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Offset")
    public @ActionOutput(name = "offset", type="byte")
           Map<String, Object> brickletRealTimeClockGetOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        byte value = ((BrickletRealTimeClockWrapper)this.handler.getDevice()).getOffset();
        
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).brickletRealTimeClockGetOffset();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRealTimeClockGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClock.Identity value = ((BrickletRealTimeClockWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).brickletRealTimeClockGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }
}
