package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRemoteSwitchV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRemoteSwitchV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Switching State")
    public @ActionOutput(name = "state", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetSwitchingState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getSwitchingState();
        
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetSwitchingState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetSwitchingState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Switch Socket A")
    public void brickletRemoteSwitchV2SwitchSocketA(
            @ActionInput(name = "houseCode") int houseCode,
            @ActionInput(name = "receiverCode") int receiverCode,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).switchSocketA(houseCode, receiverCode, switchTo);
        
    }

    public static void brickletRemoteSwitchV2SwitchSocketA(@Nullable ThingActions actions, int houseCode, int receiverCode, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2SwitchSocketA(houseCode, receiverCode, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Switch Socket B")
    public void brickletRemoteSwitchV2SwitchSocketB(
            @ActionInput(name = "address") long address,
            @ActionInput(name = "unit") int unit,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).switchSocketB(address, unit, switchTo);
        
    }

    public static void brickletRemoteSwitchV2SwitchSocketB(@Nullable ThingActions actions, long address, int unit, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2SwitchSocketB(address, unit, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Dim Socket B")
    public void brickletRemoteSwitchV2DimSocketB(
            @ActionInput(name = "address") long address,
            @ActionInput(name = "unit") int unit,
            @ActionInput(name = "dimValue") int dimValue) throws TinkerforgeException {
        ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).dimSocketB(address, unit, dimValue);
        
    }

    public static void brickletRemoteSwitchV2DimSocketB(@Nullable ThingActions actions, long address, int unit, int dimValue) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2DimSocketB(address, unit, dimValue);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Switch Socket C")
    public void brickletRemoteSwitchV2SwitchSocketC(
            @ActionInput(name = "systemCode") char systemCode,
            @ActionInput(name = "deviceCode") int deviceCode,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).switchSocketC(systemCode, deviceCode, switchTo);
        
    }

    public static void brickletRemoteSwitchV2SwitchSocketC(@Nullable ThingActions actions, char systemCode, int deviceCode, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2SwitchSocketC(systemCode, deviceCode, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Remote Configuration")
    public @ActionOutput(name = "remoteType", type="int")
           @ActionOutput(name = "minimumRepeats", type="int")
           @ActionOutput(name = "callbackEnabled", type="boolean")
           Map<String, Object> brickletRemoteSwitchV2GetRemoteConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.RemoteConfiguration value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getRemoteConfiguration();
        
        result.put("remoteType", value.remoteType);
        result.put("minimumRepeats", value.minimumRepeats);
        result.put("callbackEnabled", value.callbackEnabled);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetRemoteConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetRemoteConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Remote Status A")
    public @ActionOutput(name = "houseCode", type="int")
           @ActionOutput(name = "receiverCode", type="int")
           @ActionOutput(name = "switchTo", type="int")
           @ActionOutput(name = "repeats", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusA(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.RemoteStatusA value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getRemoteStatusA();
        
        result.put("houseCode", value.houseCode);
        result.put("receiverCode", value.receiverCode);
        result.put("switchTo", value.switchTo);
        result.put("repeats", value.repeats);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusA(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetRemoteStatusA();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Remote Status B")
    public @ActionOutput(name = "address", type="long")
           @ActionOutput(name = "unit", type="int")
           @ActionOutput(name = "switchTo", type="int")
           @ActionOutput(name = "dimValue", type="int")
           @ActionOutput(name = "repeats", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusB(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.RemoteStatusB value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getRemoteStatusB();
        
        result.put("address", value.address);
        result.put("unit", value.unit);
        result.put("switchTo", value.switchTo);
        result.put("dimValue", value.dimValue);
        result.put("repeats", value.repeats);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusB(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetRemoteStatusB();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Remote Status C")
    public @ActionOutput(name = "systemCode", type="char")
           @ActionOutput(name = "deviceCode", type="int")
           @ActionOutput(name = "switchTo", type="int")
           @ActionOutput(name = "repeats", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusC(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.RemoteStatusC value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getRemoteStatusC();
        
        result.put("systemCode", value.systemCode);
        result.put("deviceCode", value.deviceCode);
        result.put("switchTo", value.switchTo);
        result.put("repeats", value.repeats);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetRemoteStatusC(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetRemoteStatusC();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Set Repeats")
    public void brickletRemoteSwitchV2SetRepeats(
            @ActionInput(name = "repeats") int repeats) throws TinkerforgeException {
        ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).setRepeats(repeats);
        
    }

    public static void brickletRemoteSwitchV2SetRepeats(@Nullable ThingActions actions, int repeats) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2SetRepeats(repeats);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Repeats")
    public @ActionOutput(name = "repeats", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetRepeats(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getRepeats();
        
        result.put("repeats", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetRepeats(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetRepeats();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRemoteSwitchV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRemoteSwitchV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.SPITFPErrorCount value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRemoteSwitchV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitchV2.Identity value = ((BrickletRemoteSwitchV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchV2Actions) {
            return ((BrickletRemoteSwitchV2Actions) actions).brickletRemoteSwitchV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchV2Actions class.");
        }
    }
}
