package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRotaryEncoderV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRotaryEncoderV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Count")
    public @ActionOutput(name = "count", type="int")
           Map<String, Object> brickletRotaryEncoderV2GetCount(
            @ActionInput(name = "reset") boolean reset) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getCount(reset);
        
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetCount(@Nullable ThingActions actions, boolean reset) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetCount(reset);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> brickletRotaryEncoderV2IsPressed(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).isPressed();
        
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2IsPressed(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2IsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRotaryEncoderV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRotaryEncoderV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRotaryEncoderV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRotaryEncoderV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRotaryEncoderV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRotaryEncoderV2.SPITFPErrorCount value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRotaryEncoderV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRotaryEncoderV2.Identity value = ((BrickletRotaryEncoderV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }
}
