package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRotaryPoti;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRotaryPotiActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Position")
    public @ActionOutput(name = "position", type="short")
           Map<String, Object> brickletRotaryPotiGetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletRotaryPotiWrapper)this.handler.getDevice()).getPosition();
        
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryPotiGetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryPotiActions) {
            return ((BrickletRotaryPotiActions) actions).brickletRotaryPotiGetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryPotiActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletRotaryPotiGetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryPotiWrapper)this.handler.getDevice()).getAnalogValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryPotiGetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryPotiActions) {
            return ((BrickletRotaryPotiActions) actions).brickletRotaryPotiGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryPotiActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRotaryPotiGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRotaryPoti.Identity value = ((BrickletRotaryPotiWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRotaryPotiGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryPotiActions) {
            return ((BrickletRotaryPotiActions) actions).brickletRotaryPotiGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryPotiActions class.");
        }
    }
}
