package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletUVLightV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletUVLightV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get UVA")
    public @ActionOutput(name = "uva", type="int")
           Map<String, Object> brickletUVLightV2GetUVA(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getUVA();
        
        result.put("uva", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetUVA(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetUVA();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get UVB")
    public @ActionOutput(name = "uvb", type="int")
           Map<String, Object> brickletUVLightV2GetUVB(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getUVB();
        
        result.put("uvb", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetUVB(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetUVB();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get UVI")
    public @ActionOutput(name = "uvi", type="int")
           Map<String, Object> brickletUVLightV2GetUVI(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getUVI();
        
        result.put("uvi", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetUVI(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetUVI();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "integrationTime", type="int")
           Map<String, Object> brickletUVLightV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getConfiguration();
        
        result.put("integrationTime", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletUVLightV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletUVLightV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletUVLightV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletUVLightV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletUVLightV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletUVLightV2.SPITFPErrorCount value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletUVLightV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletUVLightV2.Identity value = ((BrickletUVLightV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletUVLightV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletUVLightV2Actions) {
            return ((BrickletUVLightV2Actions) actions).brickletUVLightV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletUVLightV2Actions class.");
        }
    }
}
